import re

__all__ = [
    "validate_cpf",
    "validate_cpf_mask",
]


def validate_cpf_mask(cpf: str) -> bool:
    if len(cpf) == 14:
        if cpf[3:4] == "." and cpf[7:8] == "." and cpf[11:12] == "-":
            return True
    return False


def validate_cpf(cpf: str) -> bool:
    cpf = re.sub("[^0-9]", "", str(cpf))

    invalid_list = [
        "00000000000",
        "11111111111",
        "22222222222",
        "33333333333",
        "44444444444",
        "55555555555",
        "66666666666",
        "77777777777",
        "88888888888",
        "99999999999",
    ]

    if cpf in invalid_list:
        return False

    if len(cpf) != 11:
        return False

    sum = 0
    weight = 10

    """ Calcula o primeiro digito """
    for n in range(9):
        sum = sum + int(cpf[n]) * weight
        weight = weight - 1

    check_digit = 11 - sum % 11

    if check_digit > 9:
        first_digit = 0
    else:
        first_digit = check_digit

    """ Calcula o segundo digito """
    sum = 0
    weight = 11
    for n in range(10):
        sum = sum + int(cpf[n]) * weight
        weight = weight - 1

    check_digit = 11 - sum % 11

    if check_digit > 9:
        second_digit = 0
    else:
        second_digit = check_digit

    if cpf[-2:] == "%s%s" % (first_digit, second_digit):
        return True
    return False



from typing import TYPE_CHECKING

from .field_erros import *

if TYPE_CHECKING:
    CPF = str
    CPFMask = str
    CPFDigits = str
    CNPJ = str
    CNPJDigits = str
    CNPJMask = str
else:
    from .fields.cnpj_field import *
    from .fields.cpf_field import *



import importlib

__all__ = [
    "get_representation",
    "get_pydantic_value_error",
    "get_pydantic_type_error",
]


def _get_pydantic_lib():
    try:
        pydantic_lib = importlib.import_module("pydantic")
    except ModuleNotFoundError:
        raise ValueError("Você instalou o Pydantic antes?")
    return pydantic_lib


def get_representation():
    pydantic_lib = _get_pydantic_lib()
    Representation = pydantic_lib.utils.Representation
    return Representation


def get_pydantic_value_error():
    pydantic_lib = _get_pydantic_lib()
    PydanticValueError = pydantic_lib.PydanticValueError
    return PydanticValueError


def get_pydantic_type_error():
    pydantic_lib = _get_pydantic_lib()
    PydanticTypeError = pydantic_lib.PydanticTypeError
    return PydanticTypeError



from typing import Any, Callable, Dict, Generator

from ..field_erros import (
    FieldDigitError,
    FieldInvalidError,
    FieldMaskError,
    FieldTypeError,
)
from ..validators.cnpj_validator import validate_cnpj, validate_cnpj_mask

__all__ = [
    "CNPJ",
    "CNPJMask",
    "CNPJDigits",
]


AnyCallable = Callable[..., Any]
CallableGenerator = Generator[AnyCallable, None, None]


class CNPJBase(str):
    __slots__ = ["number"]

    def __init__(self, number: str) -> None:
        self.number = number

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(type="string", format="cpf")

    @classmethod
    def validate_type(cls, value: str) -> str:
        if not isinstance(value, str):
            raise FieldTypeError()
        return value

    @classmethod
    def validate(cls, value: str) -> str:
        if not validate_cnpj(value):
            raise FieldInvalidError()
        return value


class CNPJ(CNPJBase):
    """
    Accepts string of CNPJ with or without mask.

    Attributes:
        number (str): CNPJ number.
    """

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate


class CNPJMask(CNPJBase):
    """
    Only Accepts string of CNPJ with mask.

    Attributes:
        number (str): CNPJ number.
    """

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate_mask
        yield cls.validate

    @classmethod
    def validate_mask(cls, value: str) -> str:
        if not validate_cnpj_mask(value):
            raise FieldMaskError()
        return value


class CNPJDigits(CNPJBase):
    """
    Only Accepts string of CNPJ with digits.

    Attributes:
        number (str): CNPJ number.
    """

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate_numbers
        yield cls.validate

    @classmethod
    def validate_numbers(cls, value: str) -> str:
        if not value.isdigit():
            raise FieldDigitError()
        return value



import re

__all__ = [
    "validate_cnpj",
    "validate_cnpj_mask",
]


def validate_cnpj_mask(cnpj: str) -> bool:
    if len(cnpj) == 18:
        if (
            cnpj[2:3] == "."
            and cnpj[6:7] == "."
            and cnpj[10:11] == "/"
            and cnpj[15:16] == "-"
        ):
            return True
    return False


def validate_cnpj(cnpj: str) -> bool:
    if not isinstance(cnpj, str):
        return False

    cnpj = re.sub("[^0-9]", "", cnpj)

    if len(cnpj) != 14:
        return False

    sum = 0
    weight = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]

    """ Valida o primeiro digito """
    for n in range(12):
        value = int(cnpj[n]) * weight[n]
        sum = sum + value

    check_digit = sum % 11

    if check_digit < 2:
        first_digit = 0
    else:
        first_digit = 11 - check_digit

    """ Valida o segundo digito """
    sum = 0
    weight = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
    for n in range(13):
        sum = sum + int(cnpj[n]) * weight[n]

    check_digit = sum % 11

    if check_digit < 2:
        second_digit = 0
    else:
        second_digit = 11 - check_digit

    if cnpj[-2:] == "%s%s" % (first_digit, second_digit):
        return True
    return False
